from typing import Optional

import strawberry
from strawberry import Private
from strawberry.relay import GlobalID, Node, NodeID
from strawberry.scalars import JSON
from strawberry.types import Info

from phoenix.db import models
from phoenix.server.api.context import Context
from phoenix.server.api.types.AnnotatorKind import AnnotatorKind

from .AnnotationSource import AnnotationSource
from .User import User, to_gql_user


@strawberry.type
class TraceAnnotation(Node):
    id_attr: NodeID[int]
    user_id: Private[Optional[int]]
    name: str
    annotator_kind: AnnotatorKind
    label: Optional[str]
    score: Optional[float]
    explanation: Optional[str]
    metadata: JSON
    trace_rowid: Private[Optional[int]]
    identifier: str
    source: AnnotationSource

    @strawberry.field
    async def trace_id(self) -> GlobalID:
        from phoenix.server.api.types.Trace import Trace

        return GlobalID(type_name=Trace.__name__, node_id=str(self.trace_rowid))

    @strawberry.field
    async def user(
        self,
        info: Info[Context, None],
    ) -> Optional[User]:
        if self.user_id is None:
            return None
        user = await info.context.data_loaders.users.load(self.user_id)
        if user is None:
            return None
        return to_gql_user(user)


def to_gql_trace_annotation(
    annotation: models.TraceAnnotation,
) -> TraceAnnotation:
    """
    Converts an ORM trace annotation to a GraphQL TraceAnnotation.
    """
    return TraceAnnotation(
        id_attr=annotation.id,
        user_id=annotation.user_id,
        trace_rowid=annotation.trace_rowid,
        name=annotation.name,
        annotator_kind=AnnotatorKind(annotation.annotator_kind),
        label=annotation.label,
        score=annotation.score,
        explanation=annotation.explanation,
        metadata=annotation.metadata_,
        identifier=annotation.identifier,
        source=AnnotationSource(annotation.source),
    )



"""
Authorization dependencies for FastAPI routes.

Usage:
    Use the provided dependencies (e.g., require_admin) with FastAPI's Depends to restrict access to
    certain routes.

    These dependencies will raise HTTP 403 if the user is not authorized.

    Example:

        from fastapi import APIRouter, Depends
        from phoenix.server.authorization import require_admin

        router = APIRouter()

        @router.post("/dangerous-thing", dependencies=[Depends(require_admin)])
        async def dangerous_thing(...):
            ...

    The require_admin dependency allows only admin or system users to access the route.
    It expects authentication to be enabled and the request.user to be set by the authentication.
"""

from fastapi import HTTPException, Request
from fastapi import status as fastapi_status

from phoenix.server.bearer_auth import PhoenixUser


def require_admin(request: Request) -> None:
    """
    FastAPI dependency to restrict access to admin or system users only.

    Usage:
        Add as a dependency to any route that should only be accessible by admin or system users:

            @router.post("/dangerous-thing", dependencies=[Depends(require_admin)])
            async def dangerous_thing(...):
                ...

    Behavior:
        - Allows access if the authenticated user is an admin or a system user.
        - Raises HTTP 403 Forbidden if the user is not authorized.
        - Expects authentication to be enabled and request.user to be set by the authentication.
    """
    user = getattr(request, "user", None)
    # System users have all privileges
    if not (isinstance(user, PhoenixUser) and user.is_admin):
        raise HTTPException(
            status_code=fastapi_status.HTTP_403_FORBIDDEN,
            detail="Only admin or system users can perform this action.",
        )
